@@ -22,9 +22,9 @@ sub apply_TableSpecs {
 	# Optional coderef to dynamically calculate the "open_url" and "open_url_multi"
 	$opt{get_path_code} = $opt{get_path_code} || sub {
 		my $Source = $_;
-		my $from = $Source->from;
-		$from = (split(/\./,$from,2))[1] || $from; #<-- get 'table' for both 'db.table' and 'table' format
-		my $module_name = lc('table_' . $from);
+		my $table = $Source->schema->class($Source->source_name)->table;
+		$table = (split(/\./,$table,2))[1] || $table; #<-- get 'table' for both 'db.table' and 'table' format
+		my $module_name = lc('table_' . $table);
 		my $path = '/tablespec/' . $module_name;
 	};
 	
@@ -36,9 +36,8 @@ sub apply_TableSpecs {
 		
 		my ($disp) = ($Source->primary_columns,$Source->columns);
 		
-		my $from = $Source->from;
-		$from = (split(/\./,$from,2))[1] || $from; #<-- get 'table' for both 'db.table' and 'table' format
-		my $table = $from;
+		my $table = $Source->schema->class($Source->source_name)->table;
+		$table = (split(/\./,$table,2))[1] || $table; #<-- get 'table' for both 'db.table' and 'table' format
 		
 		my %conf = (
 			title => $table,
