@@ -12,6 +12,7 @@ use Time::HiRes qw(gettimeofday tv_interval);
 use RapidApp::Data::Dmap qw(dmap);
 use URI::Escape;
 use Scalar::Util qw(looks_like_number);
+use Digest::SHA1;
 
 if($ENV{DBIC_TRACE}) {
 	debug_around 'DBIx::Class::Storage::DBI::_execute', newline => 1, stack=>20;
@@ -72,6 +73,45 @@ has 'natural_column_order', is => 'ro', isa => 'Bool', default => 1;
 
 has 'allow_restful_queries', is => 'ro', isa => 'Bool', default => 0;
 
+# Generate a param string unique to this module by URL/path. This only needs to be unique
+# among modules whose ->content may be rendered within the same request, which is only
+# being done for good measure
+has '_rst_qry_param', is => 'ro', isa => 'Str', lazy => 1, default => sub {
+  my $self = shift;
+  join('_',
+    'rst_qry',
+    substr(Digest::SHA1->new->add($self->base_url)->hexdigest, 0, 5)
+  );
+};
+sub _appl_base_params {
+  my ($self, $params) = @_;
+  my $c = $self->c;
+  
+  %{$c->req->params} = ( %{$c->req->params}, %$params );
+  
+  my $baseParams = $self->DataStore->get_extconfig_param('baseParams') || {};
+  %$baseParams = ( %$baseParams, %$params );
+  $self->DataStore->apply_extconfig( baseParams => $baseParams );
+}
+sub _appl_rst_qry {
+  my ($self, $val) = @_;
+  $self->_appl_base_params({ $self->_rst_qry_param => $val });
+}
+sub _retr_rst_qry {
+  my $self = shift;
+  my $c = RapidApp->active_request_context or return undef;
+  my $rst_qry = $c->req->params->{ $self->_rst_qry_param } or return undef;
+  
+  # Re-apply the rst_qry now to make sure there is not a caching issue
+  # in the DataStore baseParams in case the normal rest logic doesn't
+  # do this, which is the case when launched from a foreign component
+  # by setting rest_args in the stash
+  $self->_appl_rst_qry( $rst_qry );
+  
+  $rst_qry
+}
+
+
 has 'ResultSource' => (
 	is => 'ro',
 	isa => 'DBIx::Class::ResultSource',
@@ -102,8 +142,8 @@ sub _ResultSet {
 	# the order of when this is called is vitally important:
 	$self->prepare_rest_request;
 	
-	if($self->c->req->params->{rest_query}) {
-		my ($key,$val) = split(/\//,$self->c->req->params->{rest_query},2);
+	if(my $rst_qry = $self->_retr_rst_qry) {
+		my ($key,$val) = split(/\//,$rst_qry,2);
 		$Rs = $self->chain_Rs_REST($Rs,$key,$val);
 	}
 
@@ -323,15 +363,17 @@ sub prepare_rest_request {
 	# (see the req_Row and and supplied_id methods in that class) while 'rest_query' is handled by
 	# all modules with the DbicLink2 role. Need to consolidate these in DbicLink2 so this all happens in 
 	# the same place
-	my $params = $key eq $self->record_pk ? { $key => $val } : { rest_query => $key . '/' . $val };
-	%{$self->c->req->params} = ( %{$self->c->req->params}, %$params );
-	my $baseParams = $self->DataStore->get_extconfig_param('baseParams') || {};
-	%$baseParams = ( %$baseParams, %$params );
-	$self->DataStore->apply_extconfig( baseParams => $baseParams );
+  if($key eq $self->record_pk) {
+    $self->_appl_base_params({$key => $val});
+  }
+  else {
+    $self->_appl_rst_qry( join('/',$key,$val) );
+  }
 	# ---
 	
 }
 
+
 sub restGetRow {
 	my ($self,$key,$val) = @_;
 	
@@ -1091,10 +1133,10 @@ sub chain_Rs_req_quicksearch {
   my $Rs = shift || $self->_ResultSet;
   my $params = shift || $self->c->req->params;
 
-  delete $params->{query} if (defined $params->{query} and $params->{query} eq '');
-  my $query = $params->{query} or return $Rs;
+  delete $params->{qs_query} if (defined $params->{qs_query} and $params->{qs_query} eq '');
+  my $query = $params->{qs_query} or return $Rs;
 
-  my $fields = $self->param_decodeIf($params->{fields},[]);
+  my $fields = $self->param_decodeIf($params->{qs_fields},[]);
   return $Rs unless (@$fields > 0);
 
   my $attr = { join => {} };
@@ -1468,67 +1510,119 @@ sub multifilter_to_dbf {
 	return $self->multifilter_translate_cond($cond,$dbfName,$f);
 }
 
-# Dynamic map to convert supplied multifilter key names into proper
-# dbic search/key names. Can be simple key/values with values as strings,
-# or CodeRefs which can return either key/value strings, or single HashRefs
-# which will be used as the whole condition, verbatim. See multifilter_translate_cond
-# below for the values supplied to the CddeRef in '$_' (as a HashRef)
-has 'multifilter_keymap', is => 'ro', default => sub {{
-	
-  'is'              => '=',
-  'equal to'        => '=',
-  'is equal to'     => '=',
-  'exactly'         => '=',
-  'before'          => '<',
-  'less than'       => '<',
-  'greater than'    => '>',
-  'after'           => '>',
-  'not equal to'    => '!=',
-  'is not equal to' => '!=',
-	
-	"doesn't contain"     => 'not_contain',
-	'starts with'         => 'starts_with',
-	'ends with'           => 'ends_with',
-	"doesn't start with"  => 'not_starts_with',
-	"doesn't end with"    => 'not_ends_with',
-	'ends with'           => 'ends_with',
-	
-	'contains'				=> sub { ('like','%'.$_->{v}.'%') },
-	'starts_with'			=> sub { ('like',$_->{v}.'%') },
-	'ends_with'				=> sub { ('like','%'.$_->{v}) },
-	'not_contain'			=> sub { ('not like','%'.$_->{v}.'%') },
-	'not_starts_with'		=> sub { ('not like',$_->{v}.'%') },
-	'not_ends_with'			=> sub { ('not like','%'.$_->{v}) },
-	'is null'				=> sub { ('=',undef) },
-	'is empty'				=> sub { ('=','') },
-	'null_empty'			=> 'null/empty status',
-	'null/empty status' => sub {
-    if   ($_->{v} eq 'is null')       { return ('=',undef)   }
-    elsif($_->{v} eq 'is empty')      { return ('=','')      }
-    elsif($_->{v} eq 'is not null')   { return ('!=',undef)   }
-    elsif($_->{v} eq 'is not empty')  { return ('!=','')     }
-    
-    # new, simple way to handle these without needing to inline dbfName
-    elsif($_->{v} eq 'is null or empty') { return [{'='=>undef},{'='=>''}]	} #<-- arrays automatically OR
-    elsif($_->{v} eq 'is not null or empty') { return {'!='=>undef,'!='=>''} } #<-- hashes automatically AND
-    
-    ## This complexity isn't needed, and, doesn't work properly when dbfName is a ref/virtual
-    ## column. Replaced with a much more simple form above.
-    #elsif($_->{v} eq is null or empty') { return { '-or',[{ $_->{dbfName} => undef },{ $_->{dbfName} => '' }] } }
-    #elsif($_->{v} eq 'is not null or empty')     { return { '-and',[{ $_->{dbfName} => { '!=' => undef } },{ $_->{dbfName} => { '!=' =>  '' } }] } }
 
-    die "Invalid null/empty condition supplied:\n" . Dumper($_);
-  },
-	
-}};
+
+my %mf_op_alias = (
+  'is'                    => '=',
+  'equal to'              => '=',
+  'is equal to'           => '=',
+  'exactly'               => '=',
+  'before'                => '<',
+  'less than'             => '<',
+  'greater than'          => '>',
+  'after'                 => '>',
+  'not equal to'          => '!=',
+  'is not equal to'       => '!=',
+  "doesn't contain"       => 'not_contain',
+  'starts with'           => 'starts_with',
+  'ends with'             => 'ends_with',
+  "doesn't start with"    => 'not_starts_with',
+  "doesn't end with"      => 'not_ends_with',
+  'ends with'             => 'ends_with',
+
+  'is null'               => 'is_null',
+  'is empty'              => 'is_empty',
+  'is not null'           => 'not_null',
+  'is not empty'          => 'not_empty',
+  'is null or empty'      => 'null_or_empty',
+  'is not null or empty'  => 'not_null_or_empty',
+
+  'null/empty status'     => 'null_empty_status'
+);
+# This will deep recurse if there there a circular refs in %mf_op_alias above
+sub _mf_resolve_op {
+  my ($self, $op) = @_;
+  $mf_op_alias{$op} ? $self->_mf_resolve_op($mf_op_alias{$op}) : $op;
+}
+
+sub _mf_get_cond {
+  my ($self,$select,$op,$val) = @_;
+
+  $op = $self->_mf_resolve_op($op);
+
+  my $cond;
+
+  if($op eq 'contains') {
+    $cond = $self->_op_fuse($select => { like => join('','%',$val,'%') });
+  }
+  elsif($op eq 'starts_with') {
+    $cond = $self->_op_fuse($select => { like => join('',$val,'%') });
+  }
+  elsif($op eq 'ends_with') {
+    $cond = $self->_op_fuse($select => { like => join('','%',$val) });
+  }
+  elsif($op eq 'not_contain') {
+    $cond = { -or => [ # NOT LIKE -OR- NULL
+      $self->_op_fuse($select => { 'not like' => join('','%',$val,'%') }),
+      $self->_op_fuse($select => { '=' => undef }),
+    ]};
+  }
+  elsif($op eq 'not_starts_with') {
+    $cond = { -or => [ # NOT LIKE -OR- NULL
+      $self->_op_fuse($select => { 'not like' => join('',$val,'%') }),
+      $self->_op_fuse($select => { '=' => undef }),
+    ]};
+  }
+  elsif($op eq 'not_ends_with') {
+    $cond = { -or => [ # NOT LIKE -OR- NULL
+      $self->_op_fuse($select => { 'not like' => join('','%',$val) }),
+      $self->_op_fuse($select => { '=' => undef }),
+    ]};
+  }
+  elsif($op eq 'is_null') {
+    $cond = $self->_op_fuse($select => { '=' => undef });
+  }
+  elsif($op eq 'is_empty') {
+    $cond = $self->_op_fuse($select => { '=' => '' });
+  }
+  elsif($op eq 'not_null') {
+    $cond = $self->_op_fuse($select => { '!=' => undef });
+  }
+  elsif($op eq 'not_empty') {
+    $cond = $self->_op_fuse($select => { '!=' => '' });
+  }
+  elsif($op eq 'null_or_empty') {
+    $cond = { -or => [
+      $self->_op_fuse($select => { '=' => undef }),
+      $self->_op_fuse($select => { '=' => '' })
+    ]};
+  }
+  elsif($op eq 'not_null_or_empty') {
+    $cond = { -and => [
+      $self->_op_fuse($select => { '!=' => undef }),
+      $self->_op_fuse($select => { '!=' => '' })
+    ]};
+  }
+  elsif($op eq 'null_empty_status') {
+    # Re-call with with the val as the op:
+    $cond = $self->_mf_get_cond($select, $val);
+  }
+  else {
+    $cond = $self->_op_fuse($select => { $op => $val });
+  }
+
+  $cond
+}
+
+sub _op_fuse {
+  my $self = shift;
+  &_binary_op_fuser($self->sql_maker, @_)
+}
+
 
 # -- multifilter_translate_cond()
-# This method now uses _binary_op_fuser() internally to generate conditions with
-# virtual/sub-select as lhs, as is the case with virtual columns (fixes #69). 
-# There is a lot of ugly code that was written before this function was available that
-# utilized HAVING that now is not needed and should be cleaned up/unfactored
-# (such as the localized $dbf_active_conditions global crap - TODO/FIXME)
-#  see also #51 for details on the HAVING design change
+#    - refactored for #88 and #89
+#    - previously modified for #69 and #51
 sub multifilter_translate_cond {
 	my $self = shift;
 	my $cond = shift;
@@ -1541,17 +1635,14 @@ sub multifilter_translate_cond {
     ? ($dbfName->{''} => $dbfName->{-as} )
     : ($dbfName       => $dbfName        );
 
+  # -- TODO - this is legacy and needs to be investigated and removed
 	# Track in localized global:
 	push @$dbf_active_conditions, {
 		name => $as,
 		field => $field, 
 		select => clone($dbfName)
 	};
-	
-	# After changing the conditions for 'null or empty' to a simpler form, a lot of the
-	# logic in here isn't currently needed. But, leaving it in for now for reference and
-	# if more complex conditions need to be added later. Not worth the time to refactor
-	# to shrink the API (TODO: revisit this later)
+  # --
 	
 	# There should be exactly 1 key/value:
 	die "invalid multifilter condition: must have exactly 1 key/value pair:\n" . Dumper($cond) 
@@ -1563,35 +1654,8 @@ sub multifilter_translate_cond {
     $column->{multifilter_type} &&
     $column->{multifilter_type} =~ /^date/
   );
-	
-	my $map = $self->multifilter_keymap->{$k};
 
-	if(ref($map) eq 'CODE') {
-		
-		local $_ = { 
-			v 		=> $v, 
-			k 		=> $k, 
-			cond 	=> $cond, 
-			dbfName	=> $dbfName, 
-			field 	=> $field 
-		};
-		
-		my @new = &$map;
-		($k) = @new;
-		
-		return &_binary_op_fuser($self->sql_maker, $select => $k) if ref($k);
-		($k,$v) = @new if (scalar @new > 1); #<-- if 2 args were returned (this approach allows $v of undef)
-	}
-	else {
-		# If we're here, $map should be a scalar/string:
-		die "unexpected data in 'multifilter_keymap'!" if ref($map);
-	
-		# remap/recall recursively to support multi-level keyname map translations:
-		# (i.e. 'equal to' could map to 'is equal to' which could map to '=')
-		return $self->multifilter_translate_cond({ $map => $v },$dbfName,$field) if ($map);
-	}
-	
-	return &_binary_op_fuser($self->sql_maker, $select => { $k => $v });
+  return $self->_mf_get_cond($select, $k, $v);
 }
 
 
@@ -2002,9 +2066,18 @@ sub _dbiclink_update_records {
 		#die usererr rawhtml $self->make_dbic_exception_friendly($err), title => 'Database Error';
 	};
 	
-	# Perform a fresh lookup of all the records we just updated and send them back to the client:
-	my $newdata = $self->DataStore->read({ columns => [ keys %{ $arr->[0] } ], id_in => \@updated_keyvals });
-	
+  # --
+  # Perform a fresh lookup of all the records we just updated and send them back to the client:
+  delete $self->c->req->params->{ $self->_rst_qry_param } if (
+    # clear any existing rst_qry to prevent polluting the read
+    exists $self->c->req->params->{ $self->_rst_qry_param }
+  );
+  my $newdata = $self->DataStore->read({
+    columns => [ keys %{ $arr->[0] } ], 
+    id_in   => \@updated_keyvals
+  });
+  # --
+  
 	## ----------------
 	# NEW: We need to make sure the order of the returned rows matches the supplied rows;
 	# Ext's data store uses the order rather than the record ids to match. If we don't do
@@ -2270,7 +2343,9 @@ sub _dbiclink_create_records {
 	#my $Rs = $self->ResultSource->resultset;
 	my $Rs = $self->baseResultSet;
 	
-	my @req_columns = $self->get_req_columns(undef,'create_columns');
+  # create_columns turned off in 080-DataStore.js - 2014-11-24 by HV
+	#my @req_columns = $self->get_req_columns(undef,'create_columns');
+  my @req_columns = $self->get_req_columns;
 	
 	# -- current real/valid columns according to DataStore2:
 	my %cols_indx = map {$_=>1} $self->column_name_list;
@@ -2333,8 +2408,17 @@ sub _dbiclink_create_records {
 		#die usererr rawhtml $self->make_dbic_exception_friendly($err), title => 'Database Error';
 	};
 	
-	# Perform a fresh lookup of all the records we just updated and send them back to the client:
-	my $newdata = $self->DataStore->read({ columns => \@req_columns, id_in => \@updated_keyvals });
+  # --
+  # Perform a fresh lookup of all the records we just updated and send them back to the client:
+  delete $self->c->req->params->{ $self->_rst_qry_param } if (
+    # clear any existing rst_qry to prevent polluting the read
+    exists $self->c->req->params->{ $self->_rst_qry_param }
+  );
+  my $newdata = $self->DataStore->read({
+    columns => \@req_columns, 
+    id_in   => \@updated_keyvals
+  });
+  # --
 	
 	die usererr rawhtml "Unknown error; no records were created", 
 		title => 'Create Failed' unless ($newdata && $newdata->{results});
